@@ -26,6 +26,8 @@
 #include <stdbool.h>
 
 #include <dpkg/macros.h>
+#include <dpkg/error.h>
+#include <dpkg/varbuf.h>
 
 DPKG_BEGIN_DECLS
 
@@ -39,10 +41,32 @@ struct file_stat {
 	uid_t uid;
 	gid_t gid;
 	mode_t mode;
+
+	/* Used by dpkg-statoverride when parsing the database to preserve the
+	 * user and group names in case the system does not know about them. */
+	char *uname;
+	char *gname;
 };
 
+void
+file_getcwd(struct varbuf *cwd);
+
+char *
+file_realpath(const char *pathname);
+
+char *
+file_canonicalize(const char *pathname);
+
+ssize_t
+file_readlink(const char *slink, struct varbuf *content, size_t content_len);
+
+bool file_is_exec(const char *filename);
+
 void file_copy_perms(const char *src, const char *dst);
 
+int
+file_slurp(const char *filename, struct varbuf *vb, struct dpkg_error *err);
+
 enum file_lock_flags {
 	FILE_LOCK_NOWAIT,
 	FILE_LOCK_WAIT,
@@ -50,8 +74,8 @@ enum file_lock_flags {
 
 bool file_is_locked(int lockfd, const char *filename);
 void file_lock(int *lockfd, enum file_lock_flags flags, const char *filename,
-               const char *desc);
-void file_unlock(int fd, const char *desc);
+               const char *filedesc);
+void file_unlock(int fd, const char *filename, const char *filedesc);
 void file_show(const char *filename);
 
 /** @} */
