@@ -2,7 +2,7 @@
  * libdpkg - Debian packaging suite library routines
  * parsehelp.c - helpful routines for parsing and writing
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  * Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
@@ -36,35 +36,35 @@
 #include <dpkg/error.h>
 #include <dpkg/parsedump.h>
 
-static const char *
-parse_error_msg(struct parsedb_state *ps, const char *fmt)
+static DPKG_ATTR_VPRINTF(2) const char *
+parse_error_msg(struct parsedb_state *ps, const char *fmt, va_list args)
 {
-  static char msg[1024];
-  char filename[256];
+  struct varbuf *vb = &ps->errmsg;
 
-  str_escape_fmt(filename, ps->filename, sizeof(filename));
+  if (ps->pkg && ps->pkg->set->name)
+    varbuf_set_fmt(vb, _("parsing file '%s' near line %d package '%s':\n "),
+                   ps->filename, ps->lno,
+                   pkgbin_name(ps->pkg, ps->pkgbin, pnaw_nonambig));
+  else
+    varbuf_set_fmt(vb, _("parsing file '%.255s' near line %d:\n "),
+                   ps->filename, ps->lno);
 
-  if (ps->pkg && ps->pkg->set->name) {
-    char pkgname[256];
+  varbuf_add_vfmt(vb, fmt, args);
 
-    str_escape_fmt(pkgname, pkgbin_name(ps->pkg, ps->pkgbin, pnaw_nonambig),
-                   sizeof(pkgname));
-    sprintf(msg, _("parsing file '%.255s' near line %d package '%.255s':\n"
-                   " %.255s"), filename, ps->lno, pkgname, fmt);
-  } else
-    sprintf(msg, _("parsing file '%.255s' near line %d:\n"
-                   " %.255s"), filename, ps->lno, fmt);
-
-  return msg;
+  return vb->buf;
 }
 
 void
 parse_error(struct parsedb_state *ps, const char *fmt, ...)
 {
   va_list args;
+  const char *str;
 
   va_start(args, fmt);
-  ohshitv(parse_error_msg(ps, fmt), args);
+  str = parse_error_msg(ps, fmt, args);
+  va_end(args);
+
+  ohshit("%s", str);
 }
 
 void
@@ -73,10 +73,43 @@ parse_warn(struct parsedb_state *ps, const char *fmt, ...)
   va_list args;
 
   va_start(args, fmt);
-  warningv(parse_error_msg(ps, fmt), args);
+  warning("%s", parse_error_msg(ps, fmt, args));
   va_end(args);
 }
 
+void
+parse_lax_problem(struct parsedb_state *ps, enum parsedbflags flags_lax,
+                  const char *fmt, ...)
+{
+  va_list args;
+  const char *str;
+
+  va_start(args, fmt);
+  str = parse_error_msg(ps, fmt, args);
+  va_end(args);
+
+  if (ps->flags & flags_lax)
+    warning("%s", str);
+  else
+    ohshit("%s", str);
+}
+
+void
+parse_problem(struct parsedb_state *ps, const char *fmt, ...)
+{
+  va_list args;
+  const char *str;
+
+  va_start(args, fmt);
+  str = parse_error_msg(ps, fmt, args);
+  va_end(args);
+
+  if (ps->err.type == DPKG_MSG_WARN)
+    warning("%s: %s", str, ps->err.str);
+  else
+    ohshit("%s: %s", str, ps->err.str);
+}
+
 const struct fieldinfo *
 find_field_info(const struct fieldinfo *fields, const char *fieldname)
 {
@@ -104,7 +137,7 @@ find_arbfield_info(const struct arbitraryfield *arbs, const char *fieldname)
 const char *
 pkg_name_is_illegal(const char *p)
 {
-  /* FIXME: _ is deprecated, remove sometime. */
+  /* TODO: _ is deprecated, remove sometime. */
   static const char alsoallowed[] = "-+._";
   static char buf[150];
   int c;
@@ -137,7 +170,7 @@ void varbufversion
         (!version->revision || !strchr(version->revision,':'))) break;
     /* Fall through. */
   case vdew_always:
-    varbuf_printf(vb, "%u:", version->epoch);
+    varbuf_add_fmt(vb, "%u:", version->epoch);
     break;
   default:
     internerr("unknown versiondisplayepochwhen '%d'", vdew);
@@ -165,11 +198,24 @@ const char *versiondescribe
   vb= &bufs[bufnum]; bufnum++; if (bufnum == 10) bufnum= 0;
   varbuf_reset(vb);
   varbufversion(vb,version,vdew);
-  varbuf_end_str(vb);
 
   return vb->buf;
 }
 
+const char *
+versiondescribe_c(const struct dpkg_version *version,
+                  enum versiondisplayepochwhen vdew)
+{
+  struct dpkg_locale oldloc;
+  const char *str;
+
+  oldloc = dpkg_locale_switch_C();
+  str = versiondescribe(version, vdew);
+  dpkg_locale_switch_back(oldloc);
+
+  return str;
+}
+
 /**
  * Parse a version string and check for invalid syntax.
  *
@@ -214,6 +260,8 @@ parseversion(struct dpkg_version *rversion, const char *string,
 
     errno = 0;
     epoch = strtol(string, &eepochcolon, 10);
+    if (string == eepochcolon)
+      return dpkg_put_error(err, _("epoch in version is empty"));
     if (colon != eepochcolon)
       return dpkg_put_error(err, _("epoch in version is not number"));
     if (epoch < 0)
@@ -229,13 +277,19 @@ parseversion(struct dpkg_version *rversion, const char *string,
   }
   rversion->version= nfstrnsave(string,end-string);
   hyphen= strrchr(rversion->version,'-');
-  if (hyphen)
+  if (hyphen) {
     *hyphen++ = '\0';
+
+    if (*hyphen == '\0')
+      return dpkg_put_error(err, _("revision number is empty"));
+  }
   rversion->revision= hyphen ? hyphen : "";
 
   /* XXX: Would be faster to use something like cisversion and cisrevision. */
   ptr = rversion->version;
-  if (*ptr && !c_isdigit(*ptr++))
+  if (!*ptr)
+    return dpkg_put_error(err, _("version number is empty"));
+  if (!c_isdigit(*ptr++))
     return dpkg_put_warn(err, _("version number does not start with digit"));
   for (; *ptr; ptr++) {
     if (!c_isdigit(*ptr) && !c_isalpha(*ptr) && strchr(".-+~:", *ptr) == NULL)
@@ -258,29 +312,24 @@ parseversion(struct dpkg_version *rversion, const char *string,
  * @param ps The parsedb state.
  * @param version The version to parse into.
  * @param value The version string to parse from.
- * @param fmt The error format string.
+ *
+ * @retval  0 On success, and err is reset.
+ * @retval -1 On failure, and err is set accordingly.
  */
-void
+int
 parse_db_version(struct parsedb_state *ps, struct dpkg_version *version,
-                 const char *value, const char *fmt, ...)
+                 const char *value)
 {
-  struct dpkg_error err;
-  va_list args;
-  char buf[1000];
+  dpkg_error_destroy(&ps->err);
 
-  if (parseversion(version, value, &err) == 0)
-    return;
+  if (parseversion(version, value, &ps->err) == 0)
+    return 0;
 
-  va_start(args, fmt);
-  vsnprintf(buf, sizeof(buf), fmt, args);
-  va_end(args);
+  /* If not in lax mode, turn everything into an error. */
+  if (!(ps->flags & pdb_lax_version_parser))
+    ps->err.type = DPKG_MSG_ERROR;
 
-  if (err.type == DPKG_MSG_WARN && (ps->flags & pdb_lax_version_parser))
-    parse_warn(ps, "%s: %.250s", buf, err.str);
-  else
-    parse_error(ps, "%s: %.250s", buf, err.str);
-
-  dpkg_error_destroy(&err);
+  return -1;
 }
 
 void
@@ -288,9 +337,9 @@ parse_must_have_field(struct parsedb_state *ps,
                       const char *value, const char *what)
 {
   if (!value)
-    parse_error(ps, _("missing %s"), what);
+    parse_error(ps, _("missing '%s' field"), what);
   else if (!*value)
-    parse_error(ps, _("empty value for %s"), what);
+    parse_error(ps, _("empty value for '%s' field"), what);
 }
 
 void
@@ -300,9 +349,9 @@ parse_ensure_have_field(struct parsedb_state *ps,
   static const char empty[] = "";
 
   if (!*value) {
-    parse_warn(ps, _("missing %s"), what);
+    parse_warn(ps, _("missing '%s' field"), what);
     *value = empty;
   } else if (!**value) {
-    parse_warn(ps, _("empty value for %s"), what);
+    parse_warn(ps, _("empty value for '%s' field"), what);
   }
 }
